@@ -69,17 +69,43 @@ class KayobeAnsibleMixin(object):
             verbosity_args["quiet"] = True
         return verbosity_args
 
-    def run_kayobe_playbooks(self, *args, **kwargs):
+    def run_kayobe_playbooks(self, parsed_args, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.run_playbooks(*args, **kwargs)
+        return ansible.run_playbooks(parsed_args, *args, **kwargs)
 
-    def run_kayobe_playbook(self, *args, **kwargs):
+    def run_kayobe_playbook(self, parsed_args, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.run_playbook(*args, **kwargs)
+        return ansible.run_playbook(parsed_args, *args, **kwargs)
 
-    def run_kayobe_config_dump(self, *args, **kwargs):
+    def run_kayobe_config_dump(self, parsed_args, *args, **kwargs):
         kwargs.update(self._get_verbosity_args())
-        return ansible.config_dump(*args, **kwargs)
+        return ansible.config_dump(parsed_args, *args, **kwargs)
+
+    def generate_kolla_ansible_config(self, parsed_args, install=False,
+                                      service_config=True,
+                                      bifrost_config=False):
+        """Generate configuration for kolla-ansible.
+
+        :param install: If True, also install kolla-ansible.
+        :param service_config: If True, generate service configuration.
+        :param bifrost_config: If True, generate bifrost configuration.
+        """
+        # We use ignore_limit here because all of these plays execute against
+        # localhost, and are typically necessary for kolla-ansible to function
+        # correctly. Previously a number of people were caught out by using
+        # --limit and having these plays skipped.
+        tags = None if install else "config"
+        playbooks = _build_playbook_list("kolla-ansible")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags=tags,
+                                  ignore_limit=True)
+        if service_config:
+            playbooks = _build_playbook_list("kolla-openstack")
+            self.run_kayobe_playbooks(parsed_args, playbooks,
+                                      ignore_limit=True)
+        if bifrost_config:
+            playbooks = _build_playbook_list("kolla-bifrost")
+            self.run_kayobe_playbooks(parsed_args, playbooks,
+                                      ignore_limit=True)
 
 
 class KollaAnsibleMixin(object):
@@ -118,21 +144,48 @@ class KollaAnsibleMixin(object):
         return kolla_ansible.run_seed(*args, **kwargs)
 
 
-class ControlHostBootstrap(KayobeAnsibleMixin, VaultMixin, Command):
+class ControlHostBootstrap(KayobeAnsibleMixin, KollaAnsibleMixin, VaultMixin,
+                           Command):
     """Bootstrap the Kayobe control environment.
 
     * Downloads and installs Ansible roles from Galaxy.
     * Generates an SSH key for the Ansible control host, if one does not exist.
     * Installs kolla-ansible on the Ansible control host.
+    * Generates admin-openrc.sh and public-openrc.sh files when passwords.yml
+      exists.
     """
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Bootstrapping Kayobe Ansible control host")
         ansible.install_galaxy_roles(parsed_args)
         playbooks = _build_playbook_list("bootstrap")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
+
+        passwords_exist = ansible.passwords_yml_exists(parsed_args)
+        if passwords_exist:
+            # Install and generate configuration - necessary for post-deploy.
+            ka_tags = None
+        else:
+            ka_tags = "install"
         playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags=ka_tags,
+                                  ignore_limit=True)
+
+        if passwords_exist:
+            # If we are bootstrapping a control host for an existing
+            # environment, we should also generate the admin-openrc.sh and
+            # public-openrc.sh scripts that provide admin credentials.
+
+            # FIXME: Fudge to work around incorrect configuration path.
+            extra_vars = {"node_config_directory":
+                          parsed_args.kolla_config_path}
+            self.run_kolla_ansible_overcloud(parsed_args, "post-deploy",
+                                             extra_vars=extra_vars)
+            # Create an environment file for accessing the public API as the
+            # admin user.
+            playbooks = _build_playbook_list("public-openrc")
+            self.run_kayobe_playbooks(parsed_args, playbooks,
+                                      ignore_limit=True)
 
 
 class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
@@ -151,9 +204,10 @@ class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
         # Use force to upgrade roles.
         ansible.install_galaxy_roles(parsed_args, force=True)
         playbooks = _build_playbook_list("bootstrap")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
         playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install")
+        self.run_kayobe_playbooks(parsed_args, playbooks, tags="install",
+                                  ignore_limit=True)
 
 
 class ConfigurationDump(KayobeAnsibleMixin, VaultMixin, Command):
@@ -385,9 +439,7 @@ class SeedVMProvision(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.run_kayobe_playbook(parsed_args,
                                  _get_playbook_path("seed-vm-provision"))
         # Now populate the Kolla Ansible inventory.
-        self.run_kayobe_playbook(parsed_args,
-                                 _get_playbook_path("kolla-ansible"),
-                                 tags="config")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
 
 class SeedVMDeprovision(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
@@ -471,8 +523,8 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             "users", "yum", "dev-tools", "disable-selinux", "network",
             "sysctl", "ip-routing", "snat", "disable-glean", "ntp", "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
         # Run kolla-ansible bootstrap-servers.
         # This command should be run as the kayobe ansible user because at this
@@ -586,11 +638,9 @@ class SeedServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Deploying seed services")
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False,
+                                           bifrost_config=True)
 
-        playbooks = _build_playbook_list("kolla-bifrost")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
@@ -619,13 +669,13 @@ class SeedServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Upgrading seed services")
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False,
+                                           bifrost_config=True)
 
         playbooks = _build_playbook_list(
-            "kolla-bifrost",
             "seed-service-upgrade-prep")
         self.run_kayobe_playbooks(parsed_args, playbooks)
+
         self.run_kolla_ansible_seed(parsed_args, "deploy-bifrost")
         playbooks = _build_playbook_list(
             "overcloud-host-image-workaround-resolv",
@@ -716,8 +766,7 @@ class OvercloudInventoryDiscover(KayobeAnsibleMixin, VaultMixin, Command):
         self.run_kayobe_playbook(parsed_args,
                                  _get_playbook_path("ip-allocation"))
         # Now populate the Kolla Ansible inventory.
-        self.run_kayobe_playbook(parsed_args, _get_playbook_path(
-            "kolla-ansible"), tags="config")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
 
 class OvercloudIntrospectionDataSave(KayobeAnsibleMixin, VaultMixin, Command):
@@ -749,7 +798,8 @@ class OvercloudIntrospectionDataSave(KayobeAnsibleMixin, VaultMixin, Command):
             extra_vars['output_dir'] = parsed_args.output_dir
         if parsed_args.output_format:
             extra_vars['output_format'] = parsed_args.output_format
-        playbooks = _build_playbook_list("overcloud-introspection-data-save")
+        playbooks = _build_playbook_list("kolla-bifrost-hostvars",
+                                         "overcloud-introspection-data-save")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   extra_vars=extra_vars)
 
@@ -872,8 +922,8 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
             "users", "yum", "dev-tools", "disable-selinux", "network",
             "sysctl", "disable-glean", "disable-cloud-init", "ntp", "lvm")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
         # Kolla-ansible bootstrap-servers.
         # The kolla-ansible bootstrap-servers command should be run as the
@@ -993,10 +1043,9 @@ class OvercloudServiceConfigurationGenerate(KayobeAnsibleMixin,
         self.app.LOG.debug("Generating overcloud service configuration")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
+        playbooks = _build_playbook_list("swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before deployment.
@@ -1082,16 +1131,20 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         self.app.LOG.debug("Deploying overcloud services")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
+        playbooks = _build_playbook_list("swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before deployment.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
 
+        # Workaround: Stop the chrony container.
+        # TODO(mgoddard): Remove in the Train cycle.
+        playbooks = _build_playbook_list("stop-chrony")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+
         # Perform the kolla-ansible deployment.
         self.run_kolla_ansible_overcloud(parsed_args, "deploy")
 
@@ -1109,7 +1162,7 @@ class OvercloudServiceDeploy(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         # Create an environment file for accessing the public API as the admin
         # user.
         playbooks = _build_playbook_list("public-openrc")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
 
 
 class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
@@ -1139,16 +1192,20 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Reconfiguring overcloud services")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args)
 
-        playbooks = _build_playbook_list("kolla-openstack", "swift-setup")
+        playbooks = _build_playbook_list("swift-setup")
         self.run_kayobe_playbooks(parsed_args, playbooks)
 
         # Run kolla-ansible prechecks before reconfiguration.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
 
+        # Workaround: Stop the chrony container.
+        # TODO(mgoddard): Remove in the Train cycle.
+        playbooks = _build_playbook_list("stop-chrony")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+
         # Perform the kolla-ansible reconfiguration.
         self.run_kolla_ansible_overcloud(parsed_args, "reconfigure")
 
@@ -1166,7 +1223,7 @@ class OvercloudServiceReconfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         # Create an environment file for accessing the public API as the admin
         # user.
         playbooks = _build_playbook_list("public-openrc")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
 
 
 class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
@@ -1195,13 +1252,17 @@ class OvercloudServiceUpgrade(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Upgrading overcloud services")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible", "kolla-openstack")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.generate_kolla_ansible_config(parsed_args, install=True)
 
         # Run kolla-ansible prechecks before upgrade.
         if not parsed_args.skip_prechecks:
             self.run_kolla_ansible_overcloud(parsed_args, "prechecks")
 
+        # Workaround: Stop the chrony container.
+        # TODO(mgoddard): Remove in the Train cycle.
+        playbooks = _build_playbook_list("stop-chrony")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
+
         # Perform the kolla-ansible upgrade.
         self.run_kolla_ansible_overcloud(parsed_args, "upgrade")
 
@@ -1240,11 +1301,7 @@ class OvercloudServiceDestroy(KollaAnsibleMixin, KayobeAnsibleMixin,
         self.app.LOG.debug("Destroying overcloud services")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
-
-        playbooks = _build_playbook_list("kolla-openstack")
-        self.run_kayobe_playbooks(parsed_args, playbooks)
+        self.generate_kolla_ansible_config(parsed_args)
 
         # Run kolla-ansible destroy.
         extra_args = ["--yes-i-really-really-mean-it"]
@@ -1266,8 +1323,7 @@ class OvercloudContainerImagePull(KayobeAnsibleMixin, KollaAnsibleMixin,
         self.app.LOG.debug("Pulling overcloud container images")
 
         # First prepare configuration.
-        playbooks = _build_playbook_list("kolla-ansible")
-        self.run_kayobe_playbooks(parsed_args, playbooks, tags="config")
+        self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
         # Pull updated kolla container images.
         self.run_kolla_ansible_overcloud(parsed_args, "pull")
